#!/bin/bash

CURRUSER=$USER

DOCKERREPO=$(cat settings.conf | grep docker_repo= | sed 's/docker_repo=//g')
REPO=$(cat settings.conf | grep ^repo= | sed 's/repo=//g')
PATHDIGIT=$(cat settings.conf | grep ^pathDigit= | sed 's/pathDigit=//g')
PATHSVD=$(cat settings.conf | grep ^pathSVD= | sed 's/pathSVD=//g')
CURLTIMEOUT=$(cat settings.conf | grep ^curltimeout= | sed 's/curltimeout=//g')
POSTGRESTIMEOUT=$(cat settings.conf | grep ^postgresTimeout= | sed 's/postgresTimeout=//g')


##########Проверка доступа к репозиториям
checkrepo(){
if [ "$(curl -s -o .curlogs -w "%{http_code}" --connect-timeout $CURLTIMEOUT $1)" = "000" ]; then
    echo -e "\033[1;31m[-]\033[0m $1"
    echo -e "\033[1;31m[ Отменяю установку ]\033[0m"
    exit
else 
    echo -e "\033[1;32m[+]\033[0m $1"
fi
}

##########Установка docker, запись в группу docker и настройка daemon.json в зависимости от дистрибутива
checkrequements(){
    
    #######Проверка заполененности кофигурации
    echo -e "\033[1;36m----------------------------------------------------------------\033[0m"
    echo -e "Скрипт запущен под пользователем \033[1;36m$CURRUSER\033[0m"
    echo -e "\033[1;32mКонфигурация:\033[0m"
    echo "DOCKERREPO [$DOCKERREPO]"
    echo "REPO [$REPO]"
    echo "PATHDIGIT [$PATHDIGIT]"
    echo "PATHSVD [$PATHSVD]"
    echo "CURLTIMEOUT [$CURLTIMEOUT]"
    echo "POSTGRESTIMEOUT [$POSTGRESTIMEOUT]"
    echo -e "\033[1;36m----------------------------------------------------------------\033[0m"
    

    if [ -z $DOCKERREPO ] || [ -z $REPO ] || [ -z $PATHDIGIT ] || [ -z $CURLTIMEOUT ] || [ -z $PATHSVD ] || [ -z $POSTGRESTIMEOUT ]; then 
        echo "Существуют незаполненные параметры в файле settings.conf"
        echo -e "\033[1;31m[ Отменяю установку ]\033[0m"
        exit
    fi
    ##############
    ### Проверка на права в группе sudo
    if [ ! -d "$PATHDIGIT" ]; then
        sudo mkdir -p $PATHDIGIT/meta
        sudo chown -R $CURRUSER:$CURRUSER $PATHDIGIT
    fi
    if [ ! -d "$PATHDIGIT/meta" ]; then
        sudo mkdir -p $PATHDIGIT/meta
        sudo chown -R $CURRUSER:$CURRUSER $PATHDIGIT/meta
    fi
    if [ ! -d "$PATHDIGIT/meta" ]; then
        echo "Ошибка в создании каталога $PATHDIGIT/meta. Укажите верный пароль от sudo или настройте права пользователя $CURRUSER"
        exit
    fi
    #############
    ############# Создаем meta файлы
    for module in $(cat raw/modules.list); 
    do
        if [ ! -f $PATHDIGIT/meta/$module ]; then
            echo "false" > $PATHDIGIT/meta/$module; 
        fi
    done
    ##############
    echo "Получаю имя системы"
    DISTRIBID=$(cat /etc/os-release | grep '^NAME=' | sed 's/NAME=//g' | sed 's/"//g')
    echo "ID = $DISTRIBID"
    if [ "$DISTRIBID" == "Ubuntu" ]; then 
        if [ "$(dpkg -s docker-ce | grep Status | sed 's/Status: //g')" != "install ok installed" ] && [ "$(dpkg -s docker.io | grep Status | sed 's/Status: //g')" != "install ok installed" ]; then 
            echo "Docker не установлен, производится попытка установки..."
            sudo apt-get update -y; sudo apt-get upgrade -y
            sudo apt-get install \
                ca-certificates \
                curl \
                gnupg \
                lsb-release
            sudo mkdir -p /etc/apt/keyrings
            curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg
            echo \
                "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu \
                $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
            sudo apt-get update
            sudo apt-get install docker-ce docker-ce-cli containerd.io docker-compose-plugin
            sudo systemctl enable docker.service; sudo systemctl start docker.service
        fi
     elif [ "$DISTRIBID" == "Astra Linux (Orel)" ]; then
        if [ "$(dpkg -s docker.io | grep Status | sed 's/Status: //g')" != "install ok installed" ]; then 
            echo "Docker не установлен, производится попытка установки..."
            sudo apt-get update -y; sudo apt-get upgrade -y
            sudo apt-get install docker.io -y
            sudo systemctl enable docker.service; sudo systemctl start docker.service
        fi    
    elif [ "$DISTRIBID" == "ALT" ]; then
        if [ "$(rpm -qa | grep docker)" == "" ]; then
            echo "Docker не установлен, производится попытка установки..."
            sudo apt-get update -y ; sudo apt-get dist-upgrade -y
            sudo apt-get install docker-ce -y
            sudo systemctl enable docker.service; sudo systemctl start docker.service
        fi 
    elif [ "$DISTRIBID" == "RED OS" ]; then
        if [ "$(yum list --installed | grep docker-ce)" == "" ]; then
            echo "Docker не установлен, производится попытка установки..."
            sudo dnf update -y ; sudo dnf upgrade -y
            sudo dnf install docker-ce -y
            sudo systemctl enable docker.service; sudo systemctl start docker.service
        fi
    else 
        echo "[ $DISTRIBID Не входит в список поддерживаемых систем ]"
        echo "[ RED OS, ALT, Ubuntu, Astra Linux (Orel)]"
        echo -e "\033[1;31m[ Отменяю установку ]\033[0m"
        exit
    fi
    if [ ! -f /etc/docker/daemon.json ]; then
        echo "Создаю конфигурацию для docker daemon - /etc/docker/daemon.json"
        sudo mkdir -p /etc/docker
         echo '{
    "log-driver": "json-file",
    "log-opts": {
    "max-size": "1g"
    }
}' | sudo tee /etc/docker/daemon.json 
        sudo systemctl restart docker.service
    elif [ "$(cat /etc/docker/daemon.json | grep 'max-size')" == "" ]; then 
        echo "WARN: В текущей системе уже настраивалась конфигурация docker, установите значение максимального объема логов"
        echo "{"
        echo "  log-driver: json-file,"
        echo "  log-opts: {"
        echo "      max-size: 1g"
        echo "  }"
        echo "}"
    fi
    if [ "$(id -nG | grep docker)" == "" ]; then 
        echo "Пользователь не в группе docker, пытаюсь установить настройки"
        sudo usermod -aG docker $CURRUSER
        echo "[ Была произведена попытка установки или настройки docker для пользователя $CURRUSER]"
        echo "[ ОБЯЗАТЕЛЬНО ПЕРЕЗАЙДИТЕ в сессию и запустите скрипт еще раз для проверки ]"
        echo "[ При повторении ошибки пользуйтесь инструкциями для установки и настройки docker на своем дистрибутиве]"
        echo "[ UBUNTU: https://docs.docker.com/engine/install/ubuntu/ ]"
        echo "[ REDOS: https://redos.red-soft.ru/base/arm/arm-other/docker-install/ ]"
        echo "[ ALT: https://www.altlinux.org/Docker ]"
        echo "[ ASTRA: https://wiki.astralinux.ru/pages/viewpage.action?pageId=158601444 ]"
        exit
    fi
    #### Доступность ресурсов
    echo "Проверяю доступ к ресурсам"
    checkrepo "https://ghcr.io"
    checkrepo $DOCKERREPO
    checkrepo $REPO
    checkrepo "https://registry-1.docker.io/v2/"
    ####Авторизация в нашем docker-repo
    cat raw/docker/props.txt | docker login --username svd_user --password-stdin $DOCKERREPO
}
##################


######## Установка cassandra
install_cassandra(){
    echo -e "\033[1;33m[ УСТАНОВКА CASSANDRA ]\033[0m"
    echo "Начинаю загрузку образа"
    docker pull cassandra:3.11
    echo "Создаю директории"
    mkdir -p $PATHDIGIT/backups/cassandra
    mkdir -p $PATHDIGIT/bin/db_scripts/cassandra
    mkdir -p $PATHDIGIT/cassandra/conf $PATHDIGIT/cassandra/data/ $PATHDIGIT/cassandra/log
    mkdir -p $PATHDIGIT/out/cassandra/tmp
    echo "Копирую скрипты"
    cp -R -v raw/cassandra/cassandra_* $PATHDIGIT/bin/db_scripts/cassandra/
    chmod 740 $PATHDIGIT/bin/db_scripts/cassandra/cassandra_start $PATHDIGIT/bin/db_scripts/cassandra/cassandra_full_backup \
        $PATHDIGIT/bin/db_scripts/cassandra/cassandra_snapshot
    if [ "$1" == "-u" ]; then
        echo -e "\033[1;33m[ ОБНОВЛЕНО ]\033[0m"
        return 1 
    fi
    cp -r -v raw/cassandra/conf/* $PATHDIGIT/cassandra/conf
    echo "true" > $PATHDIGIT/meta/cassandra
    echo -e "\033[1;33m[ УСТАНОВЛЕНО ]\033[0m"
}
##########################

######## Установка postgres
install_postgres(){
    echo -e "\033[1;33m[ УСТАНОВКА POSTGRES ]\033[0m"
    echo "Начинаю загрузку образа"
    docker pull postgis/postgis:12-3.3
    echo "Создаю директории"
    mkdir -p $PATHDIGIT/backups/postgres
    mkdir -p $PATHDIGIT/bin/db_scripts/postgres
    mkdir -p $PATHDIGIT/postgres/data $PATHDIGIT/postgres/logs
    mkdir -p $PATHDIGIT/out/postgres
    echo "Копирую скрипты"
    cp -R -v raw/postgres/* $PATHDIGIT/bin/db_scripts/postgres/
    chmod 740 $PATHDIGIT/bin/db_scripts/postgres/postgres_start \
        $PATHDIGIT/bin/db_scripts/postgres/postgres_dump
    echo "Создаю необходимые БД"
    $PATHDIGIT/bin/db_scripts/postgres/postgres_start
    sleep $POSTGRESTIMEOUT
    # Создаю базы
    docker exec -ti digit_postgres psql -U postgres -tc "SELECT 1 FROM pg_database WHERE datname = 'ascug'" | grep -q 1 || docker exec -ti digit_postgres psql -U postgres -c "CREATE DATABASE ascug"
    docker exec -ti digit_postgres psql -U postgres -tc "SELECT 1 FROM pg_database WHERE datname = 'pros'" | grep -q 1 || docker exec -ti digit_postgres psql -U postgres -c "CREATE DATABASE pros"
    docker exec -ti digit_postgres psql -U postgres -tc "SELECT 1 FROM pg_database WHERE datname = 'meas_params'" | grep -q 1 || docker exec -ti digit_postgres psql -U postgres -c "CREATE DATABASE meas_params"
    docker exec -ti digit_postgres psql -U postgres -tc "SELECT 1 FROM pg_database WHERE datname = 'gar'" | grep -q 1 || docker exec -ti digit_postgres psql -U postgres -c "CREATE DATABASE gar"
    docker exec -ti digit_postgres psql -U postgres -tc "SELECT 1 FROM pg_database WHERE datname = 'protect_means'" | grep -q 1 || docker exec -ti digit_postgres psql -U postgres -c "CREATE DATABASE protect_means"
    # Запускаю postgres и включаю extentions
    docker exec -ti digit_postgres psql -U postgres -d pros -c "create extension postgis;"
    docker stop digit_postgres; docker rm digit_postgres
    echo "true" > $PATHDIGIT/meta/postgres
    if [ "$1" == "-u" ]; then
        echo -e "\033[1;33m[ ОБНОВЛЕНО ]\033[0m"
        return 1
    else 
         echo -e "\033[1;33m[ УСТАНОВЛЕНО ]\033[0m"
    fi
}
##########################

######## Установка цифры
install_pros(){
    echo -e "\033[1;33m[ УСТАНОВКА ИУСЦИФРА ]\033[0m"
    echo "Скачиваю образ gitlab-generic-packages"
    docker pull ghcr.io/wilyr/gitlab-generic-packages:0.1.5
    echo "Начинаю загрузку образа"
    DOCKERREPOFORMAT=$(echo $DOCKERREPO | sed -r 's/\///g' | sed 's/https://g' | sed 's/http://g')
    docker pull $DOCKERREPOFORMAT/digit_uberjar:latest
    echo "Создаю директории"
    mkdir -p $PATHDIGIT/bin/pros_scripts
    mkdir -p $PATHDIGIT/backups/pros
    mkdir -p $PATHDIGIT/app
    mkdir -p $PATHDIGIT/out/pros $PATHDIGIT/out/gitlab/cert $PATHDIGIT/out/gitlab/conf $PATHDIGIT/out/gitlab/out
    echo "Копирую скрипты"
    cp -R -v raw/pros/*  $PATHDIGIT/bin/pros_scripts/
    rm -rf $PATHDIGIT/bin/pros_scripts/conf
    chmod 740 $PATHDIGIT/bin/pros_scripts/*
    if [ "$1" == "-u" ]; then
        echo -e "\033[1;33m[ ОБНОВЛЕНО ]\033[0m"
        return 1 
    fi
    cp -R -v raw/pros/conf/* $PATHDIGIT/app/
    echo "true" > $PATHDIGIT/meta/pros
    echo -e "\033[1;33m[ УСТАНОВЛЕНО ]\033[0m"
}
##########################

############ Установка скриптов cron
install_cronjobs() {
    echo "[ УСТАНАНОВКА CRONJOBS ]"
    (crontab -l ; echo "@daily $PATHDIGIT/bin/db_scripts/postgres/postgres_dump; $PATHDIGIT/bin/db_scripts/cassandra/cassandra_snapshot") | sort - | uniq - | crontab -
    (crontab -l ; echo "@daily $PATHDIGIT/bin/pros_scripts/rm_backups;") | sort - | uniq - | crontab -
    (crontab -l ; echo "0 3 * * 6 $PATHDIGIT/bin/pros_scripts/local_update dock;") | sort - | uniq - | crontab -
    echo "true" > $PATHDIGIT/meta/cronjobs
}
##########################

############ Установка svd
install_svd(){
    DOCKERREPOFORMAT=$(echo $DOCKERREPO | sed -r 's/\///g' | sed 's/https://g' | sed 's/http://g')
    if [ ! -d "$PATHSVD" ]; then
        sudo mkdir -p $PATHSVD
        sudo chown -R $CURRUSER:$CURRUSER $PATHSVD
    fi
    if [ "$1" == hub ]; then
        mkdir -p $PATHSVD/hub_tmpl/
        docker pull $DOCKERREPOFORMAT/svd_hub:latest 
        if [ "$2" == "-u" ]; then
            cp -r -v raw/svd/hub_tmpl/*.sh $PATHSVD/hub_tmpl/
            echo -e "\033[1;33m[ ОБНОВЛЕНО ]\033[0m"
            return 1 
        fi
        cp -r -v raw/svd/hub_tmpl/* $PATHSVD/hub_tmpl/
        echo "true" > $PATHDIGIT/meta/$1
    else 
        mkdir -p $PATHSVD/svd_$1/
        docker pull $DOCKERREPOFORMAT/svd_$1:latest
        if [ "$2" == "-u" ]; then
            cp -r -v raw/svd/svd_module/*.sh $PATHSVD/svd_$1/
            echo -e "\033[1;33m[ ОБНОВЛЕНО ]\033[0m"
            return 1 
        fi
        cp -r -v raw/svd/svd_module/* $PATHSVD/svd_$1/
        echo "$1" > $PATHSVD/svd_$1/svd.module 
        echo "true" > $PATHDIGIT/meta/$1
    fi
    echo -e "\033[1;33m[ УСТАНОВЛЕНО ]\033[0m"
}

##########################

########## Сервис обработки параметров
install_service() {
    if [ "$1" == "all" ]; then
        install_service cassandra $2
        install_service postgres $2
        install_service pros $2
        install_service cronjobs $2
        return 1
    fi
    if [ "$(cat $PATHDIGIT/meta/$1)" == "true" ]; then
        if [ "$2" != "-f" ] && [ "$2" != "-u" ]; then
            echo "---"
            echo "ERROR: Модуль $1 уже устанавливался на этом компьютере. "
            echo "Используйте:"
            echo " ./install $1 -u - Обновить базовые образы и скрипты управления, данные не будут изменены"
            echo " ./install $1 -f - Полностью переусановит модуль. ДАННЫЕ БУДУТ УДАЛЕНЫ!"
            echo "---"
        elif [ "$2" == "-f" ] ; then
            if [ "$1" == "pros" ] || [ "$1" == "cassandra" ] || [ "$1" == "postgres" ] || [ "$1" == "cronjobs" ]; then
                sudo rm -rf $PATHDIGIT/$1
                install_$1
            else 
                sudo rm -rf $PATHSVD/svd_$1
                install_svd $1
            fi
        elif [ "$2" == "-u" ]; then
            if [ "$1" == "pros" ] || [ "$1" == "cassandra" ] || [ "$1" == "postgres" ] || [ "$1" == "cronjobs" ]; then
                install_$1 "-u"
            else
                install_svd $1 "-u"
            fi
        fi
    elif [ "$(cat $PATHDIGIT/meta/$1)" == "false" ]; then
        if [ "$1" == "pros" ] || [ "$1" == "cassandra" ] || [ "$1" == "postgres" ] || [ "$1" == "cronjobs" ]; then
            install_$1
        else
            install_svd $1 
        fi
    fi
}
##########################


############ Точка входа и информация о скрипте
if [ $(cat raw/modules.list | grep -ow "$1" | wc -w) != "1" ]; then
    echo -e "\033[1;36m----------------------------------------------------------------\033[0m"
    echo -e "\033[1;36m-------------------\033[0m\033[37;1;41m::Установочник ИУС ЦИФРА::\033[0m\\033[1;36m-------------------\033[0m"
    echo -e "\033[1;36m----------------------------------------------------------------\033[0m"
    echo -e "Версия: \033[1;32mv2023.02.23 \033[0m"
    echo -e "Поддерживаемые OC: \033[1;32mUbuntu 18.04-22.04, RedOs 7.3, ALT Linux 10.1, Astra Linux (Orel)]\033[0m"
    echo -e "Список модулей: \033[1;32m'./install.sh list'\033[0m"
    echo -e "Посмотреть документацию: \033[1;32m'./install.sh help'\033[0m. Управление: PageUp/PageDown, 'q' для выхода."
    echo -e "\033[1;36m----------------------------------------------------------------\033[0m"
elif [ "$1" == "help" ]; then
    less README.md
elif [ "$1" == "list" ]; then
    echo -e "\033[1;31m$(cat raw/modules.list)\033[0m"
else
    checkrequements
    install_service $1 $2
fi
##########################
